@@ -1,5 +1,6 @@
 const Applet = imports.ui.applet;
 const GLib = imports.gi.GLib;
+const Gio = imports.gi.Gio;
 const GObject = imports.gi.GObject;
 const Lang = imports.lang;
 const NM = imports.gi.NM;
@@ -21,7 +22,8 @@ const NMConnectionCategory = {
     WIRED: 'wired',
     WIRELESS: 'wireless',
     WWAN: 'wwan',
-    VPN: 'vpn'
+    VPN: 'vpn',
+    WIREGUARD: 'wireguard'
 };
 
 const NMAccessPointSecurity = {
@@ -43,6 +45,23 @@ const NM80211ApSecurityFlags = NM['80211ApSecurityFlags'];
 // (the remaining are placed into More...)
 const NUM_VISIBLE_NETWORKS = 5;
 
+var NMIface = '\
+<node> \
+  <interface name="org.freedesktop.NetworkManager"> \
+    <method name="CheckConnectivity"> \
+      <arg type="u" name="connectivity" direction="out"/> \
+    </method> \
+  </interface> \
+</node>';
+
+var NMProxy = Gio.DBusProxy.makeProxyWrapper(NMIface);
+function NMDBus(initCallback, cancellable) {
+    return new NMProxy(Gio.DBus.system,
+                       'org.freedesktop.NetworkManager',
+                       '/org/freedesktop/NetworkManager',
+                       initCallback, cancellable);
+}
+
 // shared between NMNetworkMenuItem and NMDeviceWWAN
 function signalToIcon(value) {
     if (value > 80)
@@ -242,7 +261,7 @@ NMWirelessSectionTitleMenuItem.prototype = {
     },
 
     updateForDevice: function(device) {
-		this._device = device;
+        this._device = device;
         // we show the switch
         // - if there not just one device
         // - if the switch is off
@@ -258,7 +277,7 @@ NMWirelessSectionTitleMenuItem.prototype = {
 
     activate: function(event) {
         PopupMenu.PopupSwitchMenuItem.prototype.activate.call(this, event);
-		log(this._setEnabledFunc);
+
         this._client[this._setEnabledFunc](this._switch.state);
 
         if (!this._device) {
@@ -479,7 +498,7 @@ NMDevice.prototype = {
     },
 
     setEnabled: function(enabled) {
-        // do nothing by default, we want to keep the conneciton list visible
+        // do nothing by default, we want to keep the connection list visible
         // in the majority of cases (wired, wwan, vpn)
     },
 
@@ -624,7 +643,7 @@ NMDevice.prototype = {
         if (active) {
             title = active._name;
         } else {
-            /* TRANSLATORS: this is the indication that a connection for another logged in user is active,
+            /* Translators: this is the indication that a connection for another logged in user is active,
                and we cannot access its settings (including the name) */
             title = _("Connected (private)");
         }
@@ -690,6 +709,7 @@ NMDevice.prototype = {
                     case NM.DeviceType.WIFI: return _("Wifi");
                     case NM.DeviceType.MODEM: return _("Modem");
                     case NM.DeviceType.BT: return _("Bluetooth");
+                    case NM.DeviceType.WIREGUARD: return _("WireGuard");
                     default: return "";
                 }
             }
@@ -987,6 +1007,62 @@ NMDeviceVPN.prototype = {
     controllable: true
 };
 
+function NMDeviceWIREGUARD() {
+    this._init.apply(this, arguments);
+}
+
+NMDeviceWIREGUARD.prototype = {
+    __proto__: NMDevice.prototype,
+
+    _init: function(client, device, connections) {
+        // Disable autoconnections
+        this._autoConnectionName = null;
+        this._client = client;
+        this.category = NMConnectionCategory.WIREGUARD;
+        this._type = NM.SETTING_WIREGUARD_SETTING_NAME;
+
+        NMDevice.prototype._init.call(this, client, null, [ ]);
+
+        // Tests:
+        this.category = NMConnectionCategory.WIREGUARD;
+        this._type = NM.SETTING_WIREGUARD_SETTING_NAME;
+    },
+
+    connectionValid: function(connection) {
+        return connection._type == NM.SETTING_WIREGUARD_SETTING_NAME;
+    },
+
+    get empty() {
+        return this._connections.length == 0;
+    },
+
+    get connected() {
+        return !!this._activeConnection;
+    },
+
+    setActiveConnection: function(activeConnection) {
+        if (activeConnection) {
+            activeConnection._type = NM.SETTING_WIREGUARD_SETTING_NAME;
+        }
+        NMDevice.prototype.setActiveConnection.call(this, activeConnection);
+
+        this.emit('active-connection-changed');
+    },
+
+    _shouldShowConnectionList: function() {
+        return true;
+    },
+
+    deactivate: function() {
+        if (this._activeConnection)
+            this._client.deactivate_connection(this._activeConnection, null);
+    },
+
+    statusLabel: null,
+    controllable: true
+};
+
+
 function NMDeviceWireless() {
     this._init.apply(this, arguments);
 }
@@ -1533,7 +1609,7 @@ NMDeviceWireless.prototype = {
         let name;
         let ssid = NM.utils_ssid_to_utf8(apObj.ssid.get_data());
         if (ssid) {
-            /* TRANSLATORS: this the automatic wireless connection name (including the network name) */
+            /* Translators: this the automatic wireless connection name (including the network name) */
             name = _("Auto %s").format(ssid);
         } else
             name = _("Auto wireless");
@@ -1662,8 +1738,8 @@ CinnamonNetworkApplet.prototype = {
 
         try {
             this.metadata = metadata;
-            Main.systrayManager.registerRole("network", metadata.uuid);
-            Main.systrayManager.registerRole("nm-applet", metadata.uuid);
+            Main.systrayManager.registerTrayIconReplacement("network", metadata.uuid);
+            Main.systrayManager.registerTrayIconReplacement("nm-applet", metadata.uuid);
 
             this.menuManager = new PopupMenu.PopupMenuManager(this);
             this.menu = new Applet.AppletPopupMenu(this, orientation);
@@ -1690,6 +1766,15 @@ CinnamonNetworkApplet.prototype = {
     _clientGot: function(obj, result) {
         try {
             this._client = NM.Client.new_finish(result);
+            this._v1_28_0 = Util.version_exceeds(this._client.get_version(), "1.28.0");
+
+            this._nm_proxy = null;
+
+            if (!this._v1_28_0) {
+                new NMDBus(Lang.bind(this, function(proxy, error) {
+                    this._nm_proxy = proxy;
+                }));
+            }
 
             this._statusSection = new PopupMenu.PopupMenuSection();
             this._statusItem = new PopupMenu.PopupMenuItem('', { style_class: 'popup-inactive-menu-item', reactive: false });
@@ -1749,15 +1834,32 @@ CinnamonNetworkApplet.prototype = {
             this.menu.addMenuItem(this._devices.vpn.section);
             this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
 
+            this._devices.wireguard = {
+                section: new PopupMenu.PopupMenuSection(),
+                device: new NMDeviceWIREGUARD(this._client),
+                item: new NMWiredSectionTitleMenuItem(_("WIREGUARD Connections"))
+            };
+            this._devices.wireguard.device.connect('active-connection-changed', Lang.bind(this, function() {
+                this._devices.wireguard.item.updateForDevice(this._devices.wireguard.device);
+            }));
+            this._devices.wireguard.item.updateForDevice(this._devices.wireguard.device);
+            this._devices.wireguard.section.addMenuItem(this._devices.wireguard.item);
+            this._devices.wireguard.section.addMenuItem(this._devices.wireguard.device.section);
+            this._devices.wireguard.section.actor.hide();
+            this.menu.addMenuItem(this._devices.wireguard.section);
+            this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
+
             this.menu.addSettingsAction(_("Network Settings"), 'network');
             this.menu.addAction(_("Network Connections"), Lang.bind(this, function() {
-				Util.spawnCommandLine("nm-connection-editor");
+                Util.spawnCommandLine("nm-connection-editor");
             }));
 
             this.menu.connect("open-state-changed", Lang.bind(this, this._onMenuOpenStateChanged));
 
             this._periodicTimeoutId = 0;
-            this._updateFrequencySeconds = DEFAULT_PERIODIC_UPDATE_FREQUENCY_SECONDS;
+            this._updateFrequencySeconds = 0;
+            this._checkingConnectivity = false;
+            this._lastConnectivityState = NM.ConnectivityState.UNKNOWN;
 
             this._activeConnections = [ ];
             this._connections = [ ];
@@ -1781,6 +1883,7 @@ CinnamonNetworkApplet.prototype = {
             this._ctypes[NM.SETTING_CDMA_SETTING_NAME] = NMConnectionCategory.WWAN;
             this._ctypes[NM.SETTING_GSM_SETTING_NAME] = NMConnectionCategory.WWAN;
             this._ctypes[NM.SETTING_VPN_SETTING_NAME] = NMConnectionCategory.VPN;
+            this._ctypes[NM.SETTING_WIREGUARD_SETTING_NAME] = NMConnectionCategory.WIREGUARD;
 
             this._readConnections();
             this._readDevices();
@@ -1854,12 +1957,14 @@ CinnamonNetworkApplet.prototype = {
          * later (some valid wifi hardware starts out initially unmanaged, for instance.)
          * We'll only count managed devices for evaluating what entries to display, and
          * when to show or hide section titles. */
-        for (let i = 0; i < devices.length; i++) {
-            if (devices[i].device.state == NM.DeviceState.UNMANAGED) {
-                continue;
-            }
+        if (devices) {
+            for (let i = 0; i < devices.length; i++) {
+                if (devices[i].device.state == NM.DeviceState.UNMANAGED) {
+                    continue;
+                }
 
-            vis_devices.push(devices[i]);
+                vis_devices.push(devices[i]);
+            }
         }
 
         if (vis_devices.length == 0) {
@@ -2046,7 +2151,7 @@ CinnamonNetworkApplet.prototype = {
             }
 
             if (!a._primaryDevice) {
-                if (a._type != NM.SETTING_VPN_SETTING_NAME) {
+                if (a._type != NM.SETTING_VPN_SETTING_NAME && a._type != NM.SETTING_WIREGUARD_SETTING_NAME) {
                     // find a good device to be considered primary
                     a._primaryDevice = null;
                     let devices = a.get_devices() || [ ];
@@ -2057,8 +2162,13 @@ CinnamonNetworkApplet.prototype = {
                             break;
                         }
                     }
-                } else
-                    a._primaryDevice = this._devices.vpn.device;
+                } else {
+                    if (a._type == NM.SETTING_VPN_SETTING_NAME)
+                        a._primaryDevice = this._devices.vpn.device;
+                    else {
+                        a._primaryDevice = this._devices.wireguard.device;
+                    }
+                }
 
                 if (a.state == NM.ActiveConnectionState.ACTIVATED &&
                     a._primaryDevice && a._primaryDevice._notification) {
@@ -2151,6 +2261,9 @@ CinnamonNetworkApplet.prototype = {
         if (section == NMConnectionCategory.VPN) {
             this._devices.vpn.device.checkConnection(connection);
             this._devices.vpn.section.actor.show();
+        } else if (section == NMConnectionCategory.WIREGUARD) {
+            this._devices.wireguard.device.checkConnection(connection);
+            this._devices.wireguard.section.actor.show();
         } else {
             let devices = this._devices[section].devices;
             for (let i = 0; i < devices.length; i++) {
@@ -2179,6 +2292,8 @@ CinnamonNetworkApplet.prototype = {
 
         if (!this._devices.vpn.device.empty)
             this._devices.vpn.section.actor.show();
+        if (!this._devices.wireguard.device.empty)
+            this._devices.wireguard.section.actor.show();
     },
 
     _syncNMState: function() {
@@ -2231,6 +2346,10 @@ CinnamonNetworkApplet.prototype = {
                     this._setIcon('network-vpn-acquiring');
                     this.set_applet_tooltip(_("Connecting to the VPN..."));
                     break;
+                case NMConnectionCategory.WIREGUARD:
+                    this._setIcon('network-vpn-acquiring');
+                    this.set_applet_tooltip(_("Connecting to WIREGUARD..."));
+                    break;
                 default:
                     // fallback to a generic connected icon
                     // (it could be a private connection of some other user)
@@ -2239,6 +2358,9 @@ CinnamonNetworkApplet.prototype = {
                 }
             } else {
                 let dev;
+                let limited_conn = false;
+                // let limited_conn = this._correctStateForTunnel(this._client.get_connectivity()) !== NM.ConnectivityState.FULL;
+
                 switch (mc._section) {
                 case NMConnectionCategory.WIRELESS:
                     dev = mc._primaryDevice;
@@ -2253,16 +2375,27 @@ CinnamonNetworkApplet.prototype = {
                             this._setIcon('network-wireless-connected');
                             this.set_applet_tooltip(_("Connected to the wireless network"));
                         } else {
-                            this._setIcon('network-wireless-signal-' + signalToIcon(ap.strength));
-                            this.set_applet_tooltip(_("Wireless connection") + ": " + ssidToLabel(ap.get_ssid()) + " ("+ ap.strength +"%)");
+                            if (limited_conn) {
+                                this._setIcon('network-wireless-no-route');
+                                this.set_applet_tooltip(_("Wireless connection") + ": " + ssidToLabel(ap.get_ssid()) + " " + _("(Limited connectivity)"));
+                            } else {
+                                this._setIcon('network-wireless-signal-' + signalToIcon(ap.strength));
+                                this.set_applet_tooltip(_("Wireless connection") + ": " + ssidToLabel(ap.get_ssid()) + " ("+ ap.strength +"%)");
+                            }
                         }
                     } else {
                         log('Active connection with no primary device?');
                     }
                     break;
                 case NMConnectionCategory.WIRED:
-                    this._setIcon('network-wired');
-                    this.set_applet_tooltip(_("Connected to the wired network"));
+                    if (limited_conn) {
+                        this._setIcon('network-wired-no-route');
+                        this.set_applet_tooltip(_("Connected to the wired network") + " " + _("(Limited connectivity)"));
+
+                    } else {
+                        this._setIcon('network-wired');
+                        this.set_applet_tooltip(_("Connected to the wired network"));
+                    }
                     break;
                 case NMConnectionCategory.WWAN:
                     dev = mc._primaryDevice;
@@ -2277,13 +2410,27 @@ CinnamonNetworkApplet.prototype = {
                         break;
                     }
 
-                    this._setIcon('network-cellular-signal-' + signalToIcon(dev.mobileDevice.signal_quality));
-                    this.set_applet_tooltip(_("Connected to the cellular network"));
+                    if (limited_conn) {
+                        this._setIcon('network-cellular-no-route');
+                        this.set_applet_tooltip(_("Connected to the cellular network") + " " + _("(Limited connectivity)"));
+                    } else {
+                        this._setIcon('network-cellular-signal-' + signalToIcon(dev.mobileDevice.signal_quality));
+                        this.set_applet_tooltip(_("Connected to the cellular network"));
+                    }
+
                     break;
                 case NMConnectionCategory.VPN:
+                    // Should we indicate limited connectivity for VPNs like we do above? What if the connection is to
+                    // a local machine? Need to test.
                     this._setIcon('network-vpn');
                     this.set_applet_tooltip(_("Connected to the VPN"));
                     break;
+                case NMConnectionCategory.WIREGUARD:
+                    // Should we indicate limited connectivity for WIREGUARDs like we do above? What if the connection is to
+                    // a local machine? Need to test.
+                    this._setIcon('network-vpn');
+                    this.set_applet_tooltip(_("Connected to WIREGUARD"));
+                    break;
                 default:
                     // fallback to a generic connected icon
                     // (it could be a private connection of some other user)
@@ -2292,6 +2439,10 @@ CinnamonNetworkApplet.prototype = {
                     break;
                 }
             }
+            if (this._devices.wireguard.item && this._devices.wireguard.item._switch.state) {
+                this._setIcon('network-vpn');
+                this.set_applet_tooltip(_("Connected to WIREGUARD"));
+            }
         }
         catch (e) {
             global.logError(e);
@@ -2306,6 +2457,25 @@ CinnamonNetworkApplet.prototype = {
 
     _periodicUpdateIcon: function() {
         let new_delay = this._updateIcon();
+        this._lastConnectivityState = this._client.get_connectivity();
+
+        // TODO: This can be unreliable with multiple interfaces, and libnm
+        // can be flaky with its connectivity state - on one machine of mine,
+        // I observed regular switching between full and limited connectivity,
+        // when no noticeable change in my actual ability to reach the internet.
+        // We might re-implement to be independent of libnm. For now just disable it.
+        //
+        // if (!this._checkingConnectivity) {
+        //     // https://gitlab.freedesktop.org/NetworkManager/NetworkManager/-/issues/476
+        //     if (this._v1_28_0) {
+        //         this._checkingConnectivity = true;
+        //         this._client.check_connectivity_async(null, Lang.bind(this, this._connectivityCheckCallback));
+        //     } else
+        //     if (this._nm_proxy != null) {
+        //         this._checkingConnectivity = true;
+        //         this._nm_proxy.CheckConnectivityRemote(Lang.bind(this, this._proxyConnectivityCheckCallback));
+        //     }
+        // }
 
         if (this._updateFrequencySeconds != new_delay) {
             this._restartPeriodicUpdateTimer(new_delay);
@@ -2316,6 +2486,56 @@ CinnamonNetworkApplet.prototype = {
         return GLib.SOURCE_REMOVE;
     },
 
+    _correctStateForTunnel(state) {
+        if (state !== NM.ConnectivityState.LIMITED) {
+            return state;
+        }
+
+        // if our primary connection state is 'limited', check if there's a tunnel
+        // connection active, and assume 'full' if so - this prevents reporting limited
+        // connectivity when something like PIA vpn is active. This could end up reporting
+        // a full connection when there isn't one, depending on what the tunnel is for,
+        // but NM doesn't seem to provide any way to discern a relationship between a virtual
+        // device (tunnel) and the physical device in use. The physical device is always returned
+        // as the primary.
+
+        if (this._activeConnections.some(con => con.get_connection_type() === "tun")) {
+            return NM.ConnectivityState.FULL;
+        }
+
+        if (this._activeConnections.some(con => con.get_connection_type() === "wireguard")) {
+            return NM.ConnectivityState.FULL;
+        }
+    },
+
+    _proxyConnectivityCheckCallback(new_state) {
+        let state = this._correctStateForTunnel(new_state[0]);
+
+        if (state !== this._lastConnectivityState) {
+            this._updateIcon();
+        }
+
+        this._checkingConnectivity = false;
+    },
+
+    _connectivityCheckCallback(source, result) {
+        let new_state = NM.ConnectivityState.UNKNOWN;
+
+        try {
+            new_state = source.check_connectivity_finish(result);
+        } catch (e) {
+            log(e);
+        }
+
+        let state = this._correctStateForTunnel(new_state);
+
+        if (state !== this._lastConnectivityState) {
+            this._updateIcon();
+        }
+
+        this._checkingConnectivity = false;
+    },
+
     _restartPeriodicUpdateTimer: function(new_delay) {
         if (this._periodicTimeoutId > 0) {
             Mainloop.source_remove(this._periodicTimeoutId);
@@ -2354,8 +2574,7 @@ CinnamonNetworkApplet.prototype = {
     },
 
     on_applet_removed_from_panel: function() {
-        Main.systrayManager.unregisterRole("network", this.metadata.uuid);
-        Main.systrayManager.unregisterRole("nm-applet", this.metadata.uuid);
+        Main.systrayManager.unregisterTrayIconReplacement(this.metadata.uuid);
         Main.keybindingManager.removeHotKey("network-open-" + this.instance_id);
         if (this._periodicTimeoutId){
             Mainloop.source_remove(this._periodicTimeoutId);
