@@ -272,7 +272,7 @@ class AppMenuButton {
         this.drawLabel = false;
         this.labelVisiblePref = false;
         this._signals = new SignalManager.SignalManager();
-        this.xid = global.screen.get_xwindow_for_window(metaWindow);
+        this.xid = metaWindow.get_xwindow();
         this._flashTimer = null;
 
         if (this._applet.orientation == St.Side.TOP)
@@ -386,7 +386,7 @@ class AppMenuButton {
         if (this._tooltip)
             this._tooltip.destroy();
 
-        if (this._applet.usePreview)
+        if (this._applet.windowHover == "thumbnail")
             this._tooltip = new WindowPreview(this, this.metaWindow, this._applet.previewScale, this._applet.showLabel);
         else
             this._tooltip = new Tooltips.PanelItemTooltip(this, "", this._applet.orientation);
@@ -521,7 +521,7 @@ class AppMenuButton {
         if (title.length > MAX_TEXT_LENGTH)
             title = title.substr(0, MAX_TEXT_LENGTH);
 
-        if (this._tooltip  && this._tooltip.set_text)
+        if (this._tooltip && this._applet.windowHover != "nothing" && this._tooltip.set_text)
             this._tooltip.set_text(title);
 
         if (this.metaWindow.minimized) {
@@ -553,21 +553,16 @@ class AppMenuButton {
     }
 
     _hasFocus() {
-        if (this.metaWindow.minimized)
+        if (!this.metaWindow || this.metaWindow.minimized)
             return false;
 
         if (this.metaWindow.has_focus())
             return true;
 
-        let transientHasFocus = false;
-        this.metaWindow.foreach_transient(function(transient) {
-            if (transient.has_focus()) {
-                transientHasFocus = true;
-                return false;
-            }
+        if (global.display.focus_window && this.metaWindow.is_ancestor_of_transient(global.display.focus_window))
             return true;
-        });
-        return transientHasFocus;
+
+        return false
     }
 
     onFocus() {
@@ -658,7 +653,7 @@ class AppMenuButton {
                 if (this._applet.buttonsUseEntireSpace) {
                     let [lminSize, lnaturalSize] = this._label.get_preferred_width(forHeight);
                     let spacing = this.actor.get_theme_node().get_length('spacing');
-                    alloc.natural_size = Math.max(150 * global.ui_scale,
+                    alloc.natural_size = Math.max(this._applet.buttonWidth * global.ui_scale,
                             naturalSize + spacing + lnaturalSize);
                 } else {
                     alloc.natural_size = this._applet.buttonWidth * global.ui_scale;
@@ -1051,15 +1046,15 @@ class CinnamonWindowListApplet extends Applet.Applet {
         this.settings.bind("button-width", "buttonWidth", this._refreshAllItems);
         this.settings.bind("buttons-use-entire-space", "buttonsUseEntireSpace", this._refreshAllItems);
         this.settings.bind("panel-show-label", "showLabelPanel", this._updateLabels);
-        this.settings.bind("window-preview", "usePreview", this._onPreviewChanged);
+        this.settings.bind("window-hover", "windowHover", this._onPreviewChanged);
         this.settings.bind("window-preview-show-label", "showLabel", this._onPreviewChanged);
         this.settings.bind("window-preview-scale", "previewScale", this._onPreviewChanged);
         this.settings.bind("last-window-order", "lastWindowOrder", null);
 
         this.signals.connect(global.display, 'window-created', this._onWindowAddedAsync, this);
-        this.signals.connect(global.screen, 'window-monitor-changed', this._onWindowMonitorChanged, this);
-        this.signals.connect(global.screen, 'window-workspace-changed', this._onWindowWorkspaceChanged, this);
-        this.signals.connect(global.screen, 'window-skip-taskbar-changed', this._onWindowSkipTaskbarChanged, this);
+        this.signals.connect(global.display, 'window-monitor-changed', this._onWindowMonitorChanged, this);
+        this.signals.connect(global.display, 'window-workspace-changed', this._onWindowWorkspaceChanged, this);
+        this.signals.connect(global.display, 'window-skip-taskbar-changed', this._onWindowSkipTaskbarChanged, this);
         this.signals.connect(Main.panelManager, 'monitors-changed', this._updateWatchedMonitors, this);
         this.signals.connect(global.window_manager, 'switch-workspace', this._refreshAllItems, this);
         this.signals.connect(Cinnamon.WindowTracker.get_default(), "window-app-changed", this._onWindowAppChanged, this);
@@ -1157,16 +1152,16 @@ class CinnamonWindowListApplet extends Applet.Applet {
         this.manager.set_spacing(spacing * global.ui_scale);
     }
 
-    _onWindowAddedAsync(screen, metaWindow, monitor) {
-        Mainloop.timeout_add(20, Lang.bind(this, this._onWindowAdded, screen, metaWindow, monitor));
+    _onWindowAddedAsync(display, metaWindow, monitor) {
+        Mainloop.timeout_add(20, Lang.bind(this, this._onWindowAdded, display, metaWindow, monitor));
     }
 
-    _onWindowAdded(screen, metaWindow, monitor) {
+    _onWindowAdded(display, metaWindow, monitor) {
         if (this._shouldAdd(metaWindow))
             this._addWindow(metaWindow, false);
     }
 
-    _onWindowMonitorChanged(screen, metaWindow, monitor) {
+    _onWindowMonitorChanged(display, metaWindow, monitor) {
         if (this._shouldAdd(metaWindow))
             this._addWindow(metaWindow, false);
         else
@@ -1180,7 +1175,7 @@ class CinnamonWindowListApplet extends Applet.Applet {
             this._refreshItem(window);
     }
 
-    _onWindowWorkspaceChanged(screen, metaWindow, metaWorkspace) {
+    _onWindowWorkspaceChanged(display, metaWindow, metaWorkspace) {
         this._refreshItemByMetaWindow(metaWindow);
     }
 
@@ -1188,13 +1183,13 @@ class CinnamonWindowListApplet extends Applet.Applet {
         this._refreshItemByMetaWindow(metaWindow);
     }
 
-    _onWindowSkipTaskbarChanged(screen, metaWindow) {
+    _onWindowSkipTaskbarChanged(display, metaWindow) {
         if (metaWindow && metaWindow.is_skip_taskbar()) {
             this._removeWindow(metaWindow);
             return;
         }
 
-        this._onWindowAdded(screen, metaWindow, 0);
+        this._onWindowAdded(display, metaWindow, 0);
     }
 
     _updateAttentionGrabber() {
