@@ -9,12 +9,13 @@ gettext.install("cinnamon", "/usr/share/locale")
 class BaseChooserButton(Gtk.Button):
     def __init__ (self, has_button_label=False):
         super(BaseChooserButton, self).__init__()
+        self.has_button_label = has_button_label
         self.set_valign(Gtk.Align.CENTER)
         self.menu = Gtk.Menu()
         self.button_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=2)
         self.button_image = Gtk.Image()
         self.button_box.add(self.button_image)
-        if has_button_label:
+        if self.has_button_label:
             self.button_label = Gtk.Label()
             self.button_box.add(self.button_label)
         self.add(self.button_box)
@@ -118,13 +119,11 @@ class PictureChooserButton(BaseChooserButton):
 
         try:
             pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(path, w, h)
+            if pixbuf:
+                return Gdk.cairo_surface_create_from_pixbuf(pixbuf, self.scale)
         except GLib.Error as e:
             print("Could not load thumbnail file '%s': %s" % (path, e.message))
-
-        if pixbuf:
-            return Gdk.cairo_surface_create_from_pixbuf(pixbuf, self.scale)
-        else:
-            return None
+        return None
 
     def set_picture_from_file (self, path):
         surface = self.create_scaled_surface(path)
@@ -134,7 +133,8 @@ class PictureChooserButton(BaseChooserButton):
             self.button_image.set_from_icon_name("user-generic", Gtk.IconSize.BUTTON)
 
     def set_button_label(self, label):
-        self.button_label.set_markup(label)
+        if self.has_button_label:
+            self.button_label.set_markup(label)
 
     def _on_picture_selected(self, menuitem, path, callback, id=None):
         if id is not None:
@@ -397,10 +397,10 @@ class TimeChooserDialog(Gtk.Dialog):
             self.labels[ttype] = Gtk.Label(self.markup(self.time[ttype]), use_markup=True)
             grid.attach(self.labels[ttype], column, 2, 1, 1)
 
-            up_button = Gtk.Button.new_from_icon_name('pan-up-symbolic', 6)
-            down_button = Gtk.Button.new_from_icon_name('pan-down-symbolic', 6)
-            up_button.set_relief(2)
-            down_button.set_relief(2)
+            up_button = Gtk.Button.new_from_icon_name('pan-up-symbolic', Gtk.IconSize.DIALOG)
+            down_button = Gtk.Button.new_from_icon_name('pan-down-symbolic', Gtk.IconSize.DIALOG)
+            up_button.set_relief(Gtk.ReliefStyle.NONE)
+            down_button.set_relief(Gtk.ReliefStyle.NONE)
             grid.attach(up_button, column, 1, 1, 1)
             grid.attach(down_button, column, 3, 1, 1)
             up_button.connect('clicked', self.shift_time, ttype, 1)
