@@ -9,6 +9,7 @@ from gi.repository import Gio, Gtk
 
 from KeybindingWidgets import CellRendererKeybinding
 from SettingsWidgets import SidePage
+from bin import util
 from xapp.GSettingsWidgets import *
 
 gettext.install("cinnamon", "/usr/share/locale")
@@ -43,7 +44,8 @@ CATEGORIES = [
     [_("Sound and Media"),  "media",            None,       "applications-multimedia"],
     [_("Quiet Keys"),           "media-quiet",      "media",        None],
     [_("Universal Access"), "accessibility",    None,       "preferences-desktop-accessibility"],
-    [_("Custom Shortcuts"), "custom",           None,       "cinnamon-panel-launcher"]
+    [_("Custom Shortcuts"), "custom",           None,       "cinnamon-panel-launcher"],
+    [_("Pointer"),          "pointer",          "general",  None]
 ]
 
 KEYBINDINGS = [
@@ -62,6 +64,9 @@ KEYBINDINGS = [
     [_("Run dialog"), MUFFIN_KEYBINDINGS_SCHEMA, "panel-run-dialog", "general"],
     # General - Troubleshooting
     [_("Toggle Looking Glass"), CINNAMON_SCHEMA, "looking-glass-keybinding", "trouble"],
+    # General - Pointer
+    [_("Move pointer to the next monitor"), CINNAMON_SCHEMA, "pointer-next-monitor", "pointer"],
+    [_("Move pointer to the previous monitor"), CINNAMON_SCHEMA, "pointer-previous-monitor", "pointer"],
     # Windows
     [_("Maximize window"), MUFFIN_KEYBINDINGS_SCHEMA, "maximize", "windows"],
     [_("Unmaximize window"), MUFFIN_KEYBINDINGS_SCHEMA, "unmaximize", "windows"],
@@ -425,18 +430,20 @@ class Module:
             vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
             vbox.set_border_width(6)
             vbox.set_spacing(6)
-            self.sidePage.stack.add_titled(vbox, "layouts", _("Layouts"))
-            try:
-                widget = self.sidePage.content_box.c_manager.get_c_widget("region")
-            except:
-                widget = None
 
-            if widget is not None:
-                cheat_box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 2)
-                cheat_box.pack_start(widget, True, True, 2)
-                cheat_box.set_vexpand(False)
-                widget.show()
-                vbox.pack_start(cheat_box, True, True, 0)
+            if util.get_session_type() != "wayland":
+                self.sidePage.stack.add_titled(vbox, "layouts", _("Layouts"))
+                try:
+                    widget = self.sidePage.content_box.c_manager.get_c_widget("region")
+                except:
+                    widget = None
+
+                if widget is not None:
+                    cheat_box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 2)
+                    cheat_box.pack_start(widget, True, True, 2)
+                    cheat_box.set_vexpand(False)
+                    widget.show()
+                    vbox.pack_start(cheat_box, True, True, 0)
 
             self.kb_search_entry.grab_focus()
 
