@@ -2,7 +2,7 @@
 
 import gi
 gi.require_version('Notify', '0.7')
-from gi.repository import Notify
+from gi.repository import Gio, Notify
 
 from SettingsWidgets import SidePage
 from xapp.GSettingsWidgets import *
@@ -29,6 +29,12 @@ MEDIA_KEYS_OSD_SIZES = [
     ("large", _("Large"))
 ]
 
+NOTIFICATION_DISPLAY_SCREENS = [
+    ("primary-screen", _("Primary monitor")),
+    ("active-screen", _("Active monitor")),
+    ("fixed-screen", _("The monitor specified below"))
+]
+
 
 class Module:
     name = "notifications"
@@ -62,6 +68,14 @@ class Module:
         switch = GSettingsSwitch(_("Show notifications on the bottom side of the screen"), "org.cinnamon.desktop.notifications", "bottom-notifications")
         settings.add_reveal_row(switch, "org.cinnamon.desktop.notifications", "display-notifications")
 
+        combo = GSettingsComboBox(_("Monitor to use for displaying notifications"), "org.cinnamon.desktop.notifications", "notification-screen-display", NOTIFICATION_DISPLAY_SCREENS)
+        settings.add_reveal_row(combo, "org.cinnamon.desktop.notifications", "display-notifications")
+
+        spin = GSettingsSpinButton(_("Monitor"), "org.cinnamon.desktop.notifications", "notification-fixed-screen", None, 1, 13, 1)
+        settings.add_reveal_row(spin)
+        spin.revealer.settings = Gio.Settings("org.cinnamon.desktop.notifications")
+        spin.revealer.settings.bind_with_mapping("notification-screen-display", spin.revealer, "reveal-child", Gio.SettingsBindFlags.GET, lambda option: option == "fixed-screen", None)
+
         spin = GSettingsSpinButton(_("Notification duration"), "org.cinnamon.desktop.notifications", "notification-duration", _("seconds"), 1, 60, 1, 1)
         settings.add_reveal_row(spin, "org.cinnamon.desktop.notifications", "display-notifications")
 
