@@ -11,6 +11,7 @@
 
 const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
+const Gtk = imports.gi.Gtk;
 const GObject = imports.gi.GObject;
 const Gir = imports.gi.GIRepository;
 const Mainloop = imports.mainloop;
@@ -348,7 +349,7 @@ function fixupPCIDescription(desc) {
         }
     }
 
-    /* Attmept to shorten ID by ignoring certain individual words */
+    /* Attempt to shorten ID by ignoring certain individual words */
     let words = desc.split(' ');
     let out = [ ];
     for (let i = 0; i < words.length; i++) {
@@ -458,102 +459,6 @@ function queryCollection(collection, query, indexOnly = false) {
     return indexOnly ? -1 : null;
 }
 
-/**
- * findIndex:
- * @array (array): Array to be iterated.
- * @callback (function): The function to call on every iteration,
- * should return a boolean value.
- *
- * Returns (number): the index of @array, else -1.
- */
-function findIndex(array, callback) {
-    for (let i = 0, len = array.length; i < len; i++) {
-        if (array[i] && callback(array[i], i, array)) {
-            return i;
-        }
-    }
-    return -1;
-}
-
-/**
- * find:
- * @array (array): Array to be iterated.
- * @callback (function): The function to call on every iteration,
- * should return a boolean value.
- *
- * Returns (any): Returns the matched element, else null.
- */
-function find(arr, callback) {
-    for (let i = 0, len = arr.length; i < len; i++) {
-        if (callback(arr[i], i, arr)) {
-            return arr[i];
-        }
-    }
-    return null;
-};
-
-/**
- * each:
- * @array (array|object): Array or object to be iterated.
- * @callback (function): The function to call on every iteration.
- *
- * Iteratee functions may exit iteration early by explicitly returning false.
- */
-function each(obj, callback) {
-    if (Array.isArray(obj)) {
-        for (let i = 0, len = obj.length; i < len; i++) {
-            if (callback(obj[i], i) === false) {
-                return;
-            }
-        }
-    } else {
-        let keys = Object.keys(obj);
-        for (let i = 0, len = keys.length; i < len; i++) {
-            let key = keys[i];
-            callback(obj[key], key);
-        }
-    }
-};
-
-/**
- * filter:
- * @array (array): Array to be iterated.
- * @callback (function): The function to call on every iteration.
- *
- * Returns (array): Returns the new filtered array.
- */
-function filter(arr, callback) {
-    let result = [];
-    for (let i = 0, len = arr.length; i < len; i++) {
-        if (callback(arr[i], i, arr)) {
-            result.push(arr[i]);
-        }
-    }
-    return result;
-};
-
-/**
- * map:
- * @array (array): Array to be iterated.
- * @callback (function): The function to call on every iteration.
- *
- * Returns (array): Returns the new mapped array.
- */
-function map(arr, callback) {
-    if (arr == null) {
-        return [];
-    }
-
-    let len = arr.length;
-    let out = Array(len);
-
-    for (let i = 0; i < len; i++) {
-        out[i] = callback(arr[i], i, arr);
-    }
-
-    return out;
-};
-
 /**
  * tryFn:
  * @callback (function): Function to wrap in a try-catch block.
@@ -716,7 +621,7 @@ const FastObject = function(o) {
 }
 FastObject();
 function toFastProperties(obj) {
-    each(obj, function(value) {
+    Object.values(obj).forEach( value => {
         if (value && !Array.isArray(value)) FastObject(value);
     });
 };
@@ -866,3 +771,27 @@ function getDesktopActionIcon(action) {
     else return null;
 }
 
+/**
+ * splitByGlyph:
+ * @str (string): The string to be converted
+ *
+ * Converts a string, possibly containing multiple codepoint unicode characters (e.g. emoji), into
+ * an array of unicode graphemes clusters (glyphs). For example: "👩‍👩‍👧‍👧😃".length === 13, but splitByGlyph("👩‍👩‍👧‍👧😃")
+ * returns ["👩‍👩‍👧‍👧", "😃"] which is length 2.
+ * 
+ * Returns (array): Array of unicode grapheme clusters (glyphs).
+ */
+function splitByGlyph(str) {
+    const glyphs = [];
+    const buffer = new Gtk.TextBuffer();
+    buffer.set_text(str, -1);
+    const iter = buffer.get_start_iter();
+    const iter2 = buffer.get_start_iter();
+    
+    while (!iter2.is_end()) {
+        iter2.forward_cursor_position();
+        glyphs.push(buffer.get_text(iter, iter2, false));
+        iter.forward_cursor_position();
+    }
+    return glyphs;
+}
