@@ -466,6 +466,13 @@ XletSettingsBase.prototype = {
 
     _getValue: function(key) {
         let value = this.settingsData[key].value;
+
+        if (value === undefined) {
+            global.logWarning(`No value field in setting '${key}' for ${this.uuid}. Using default value.`);
+
+            value = this.settingsData[key].default;
+        }
+
         return value;
     },
 
@@ -682,10 +689,21 @@ XletSettingsBase.prototype = {
     },
 
     _ensureSettingsFiles: function() {
-        let configPath = [GLib.get_home_dir(), ".cinnamon", "configs", this.uuid].join("/");
+        let configPath = [GLib.get_user_config_dir(), "cinnamon", "spices", this.uuid].join("/");
         let configDir = Gio.file_new_for_path(configPath);
         if (!configDir.query_exists(null)) configDir.make_directory_with_parents(null);
-        this.file = configDir.get_child(this.instanceId + ".json");
+
+        let configFile = configDir.get_child(this.instanceId + ".json")
+
+        let oldConfigDir = Gio.file_new_for_path([GLib.get_home_dir(), ".cinnamon", "configs", this.uuid].join("/"));
+        let oldConfigFile = oldConfigDir.get_child(this.instanceId + ".json");
+
+        // We only use the config under the old path if it's the only one for backwards compatibility
+        if (oldConfigFile.query_exists(null) && !configFile.query_exists(null))
+            this.file = oldConfigFile;
+        else 
+            this.file = configFile;
+
         this.monitor = this.file.monitor_file(Gio.FileMonitorFlags.NONE, null);
 
         // If the settings have already been installed previously we need to check if the schema
@@ -697,7 +715,6 @@ XletSettingsBase.prototype = {
                 // Some users with a little bit of know-how may be able to fix the file if it's not too corrupted. Is it worth it to give an option to skip this step?
                 global.logError(e);
                 global.logError("Failed to parse file "+this.file.get_path()+". Attempting to rebuild the settings file for "+this.uuid+".");
-                Main.notify("Unfortunately the settings file for "+this.uuid+" seems to be corrupted. Your settings have been reset to default.");
             }
         }
 
