@@ -6,7 +6,6 @@
 #include <gio/gio.h>
 #include <clutter/clutter.h>
 #include <meta/meta-window-actor.h>
-#include <libsoup/soup.h>
 
 G_BEGIN_DECLS
 
@@ -27,18 +26,12 @@ char    *cinnamon_util_format_date                (const char       *format,
 ClutterModifierType
          cinnamon_get_event_state                 (ClutterEvent     *event);
 
-void     cinnamon_write_soup_message_to_stream    (GOutputStream    *stream,
-                                                SoupMessage      *message,
-                                                GError          **error);
-
 gboolean cinnamon_write_string_to_stream          (GOutputStream    *stream,
                                                 const char       *str,
                                                 GError          **error);
 
 char    *cinnamon_get_file_contents_utf8_sync     (const char       *path,
                                                 GError          **error);
-
-gboolean cinnamon_get_gpu_offload_supported        (void);
 /**
  * CinnamonFileContentsCallback:
  * @utf8_contents: The contents of the file
