@@ -198,14 +198,14 @@ remove_capslock_feedback (StEntry *entry)
 }
 
 static void
-keymap_state_changed (GdkKeymap *keymap,
+keymap_state_changed (ClutterKeymap *keymap,
                       gpointer   user_data)
 {
   StEntry *entry = ST_ENTRY (user_data);
 
   if (clutter_text_get_password_char (CLUTTER_TEXT (entry->priv->entry)) != 0)
     {
-      if (gdk_keymap_get_caps_lock_state (keymap))
+      if (clutter_keymap_get_caps_lock_state (keymap))
         show_capslock_feedback (entry);
       else
         remove_capslock_feedback (entry);
@@ -217,7 +217,8 @@ st_entry_dispose (GObject *object)
 {
   StEntry *entry = ST_ENTRY (object);
   StEntryPrivate *priv = entry->priv;
-  GdkKeymap *keymap;
+  ClutterKeymap *keymap;
+  ClutterSeat *seat;
 
   if (priv->blink_timeout)
     {
@@ -225,7 +226,8 @@ st_entry_dispose (GObject *object)
       priv->blink_timeout = 0;
     }
 
-  keymap = gdk_keymap_get_for_display (gdk_display_get_default ());
+  seat = clutter_backend_get_default_seat (clutter_get_default_backend ());
+  keymap = clutter_seat_get_keymap (seat);
   g_signal_handlers_disconnect_by_func (keymap, keymap_state_changed, entry);
 
   G_OBJECT_CLASS (st_entry_parent_class)->dispose (object);
@@ -631,7 +633,8 @@ clutter_text_focus_in_cb (ClutterText  *text,
 {
   StEntry *entry = ST_ENTRY (actor);
   StEntryPrivate *priv = entry->priv;
-  GdkKeymap *keymap;
+  ClutterSeat *seat;
+  ClutterKeymap *keymap;
 
   /* remove the hint if visible */
   if (priv->hint && priv->hint_visible)
@@ -641,7 +644,9 @@ clutter_text_focus_in_cb (ClutterText  *text,
       clutter_text_set_text (text, "");
     }
 
-  keymap = gdk_keymap_get_for_display (gdk_display_get_default ());
+  seat = clutter_backend_get_default_seat (clutter_get_default_backend ());
+  keymap = clutter_seat_get_keymap (seat);
+
   keymap_state_changed (keymap, entry);
   g_signal_connect (keymap, "state-changed",
                     G_CALLBACK (keymap_state_changed), entry);
@@ -659,7 +664,8 @@ clutter_text_focus_out_cb (ClutterText  *text,
 {
   StEntry *entry = ST_ENTRY (actor);
   StEntryPrivate *priv = entry->priv;
-  GdkKeymap *keymap;
+  ClutterKeymap *keymap;
+  ClutterSeat *seat;
 
   st_widget_remove_style_pseudo_class (ST_WIDGET (actor), "focus");
 
@@ -674,7 +680,8 @@ clutter_text_focus_out_cb (ClutterText  *text,
   st_entry_check_cursor_blink (entry);
   remove_capslock_feedback (entry);
 
-  keymap = gdk_keymap_get_for_display (gdk_display_get_default ());
+  seat = clutter_backend_get_default_seat (clutter_get_default_backend ());
+  keymap = clutter_seat_get_keymap (seat);
   g_signal_handlers_disconnect_by_func (keymap, keymap_state_changed, entry);
 }
 
