@@ -61,7 +61,9 @@ struct _StIconPrivate
   CoglPipeline  *shadow_pipeline;
 
   StShadow     *shadow_spec;
-  ClutterSize   shadow_size;
+
+  gfloat        shadow_width;
+  gfloat        shadow_height;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (StIcon, st_icon, ST_TYPE_WIDGET)
@@ -189,12 +191,13 @@ st_icon_finalize (GObject *gobject)
 }
 
 static void
-st_icon_paint (ClutterActor *actor)
+st_icon_paint (ClutterActor        *actor,
+               ClutterPaintContext *paint_context)
 {
   StIcon *icon = ST_ICON (actor);
   StIconPrivate *priv = icon->priv;
 
-  st_widget_paint_background (ST_WIDGET (actor));
+  st_widget_paint_background (ST_WIDGET (actor), paint_context);
 
   if (priv->icon_texture)
     {
@@ -202,7 +205,7 @@ st_icon_paint (ClutterActor *actor)
       if (priv->shadow_pipeline)
         {
           ClutterActorBox allocation;
-          CoglFramebuffer *fb = cogl_get_draw_framebuffer();
+          CoglFramebuffer *fb = clutter_paint_context_get_framebuffer (paint_context);
 
           clutter_actor_get_allocation_box (priv->icon_texture, &allocation);
 
@@ -213,7 +216,7 @@ st_icon_paint (ClutterActor *actor)
                                          clutter_actor_get_paint_opacity (priv->icon_texture));
         }
 
-      clutter_actor_paint (priv->icon_texture);
+      clutter_actor_paint (priv->icon_texture, paint_context);
     }
 }
 
@@ -316,7 +319,8 @@ st_icon_clear_shadow_pipeline (StIcon *icon)
   StIconPrivate *priv = icon->priv;
 
   g_clear_pointer (&priv->shadow_pipeline, cogl_object_unref);
-  clutter_size_init (&priv->shadow_size, 0, 0);
+  priv->shadow_width = 0.0;
+  priv->shadow_height = 0.0;
 }
 
 static void
@@ -333,8 +337,8 @@ st_icon_update_shadow_pipeline (StIcon *icon)
       clutter_actor_box_get_size (&box, &width, &height);
 
       if (priv->shadow_pipeline == NULL ||
-          priv->shadow_size.width != width ||
-          priv->shadow_size.height != height)
+          priv->shadow_width != width ||
+          priv->shadow_height != height)
         {
           st_icon_clear_shadow_pipeline (icon);
 
@@ -343,18 +347,21 @@ st_icon_update_shadow_pipeline (StIcon *icon)
                                                    priv->icon_texture);
 
           if (priv->shadow_pipeline)
-            clutter_size_init (&priv->shadow_size, width, height);
+          {
+            priv->shadow_width = width;
+            priv->shadow_height = height;
+          }
         }
     }
 }
 
-static void
-on_pixbuf_changed (ClutterTexture *texture,
-                   StIcon         *icon)
-{
-  st_icon_clear_shadow_pipeline (icon);
-  clutter_actor_queue_redraw (CLUTTER_ACTOR (icon));
-}
+// static void
+// on_pixbuf_changed (ClutterTexture *texture,
+//                    StIcon         *icon)
+// {
+//   st_icon_clear_shadow_pipeline (icon);
+//   clutter_actor_queue_redraw (CLUTTER_ACTOR (icon));
+// }
 
 static void
 on_texture_file_cb (StTextureCache *cache,
@@ -390,8 +397,8 @@ st_icon_finish_update (StIcon *icon)
       st_icon_clear_shadow_pipeline (icon);
 
       /* "pixbuf-change" is actually a misnomer for "texture-changed" */
-      g_signal_connect (priv->icon_texture, "pixbuf-change",
-                        G_CALLBACK (on_pixbuf_changed), icon);
+      // g_signal_connect (priv->icon_texture, "pixbuf-change",
+      //                   G_CALLBACK (on_pixbuf_changed), icon);
     }
 }
 
@@ -415,9 +422,6 @@ st_icon_update (StIcon *icon)
   StIconPrivate *priv = icon->priv;
   StThemeNode *theme_node;
   StTextureCache *cache;
-  gint scale;
-  ClutterActor *stage;
-  StThemeContext *context;
 
   if (priv->pending_texture)
     {
@@ -431,11 +435,7 @@ st_icon_update (StIcon *icon)
   if (theme_node == NULL)
     return;
 
-  stage = clutter_actor_get_stage (CLUTTER_ACTOR (icon));
-  context = st_theme_context_get_for_stage (CLUTTER_STAGE (stage));
-  g_object_get (context, "scale-factor", &scale, NULL);
-
-  priv->icon_scale = scale;
+  priv->icon_scale = st_theme_context_get_scale_for_stage ();
 
   cache = st_texture_cache_get_default ();
   if (priv->gicon)
@@ -489,15 +489,11 @@ st_icon_update_icon_size (StIcon *icon)
   else if (priv->theme_icon_size > 0)
     {
       gint scale;
-      ClutterActor *stage;
-      StThemeContext *context;
 
       /* The theme will give us an already-scaled size, so we
        * undo it here, as priv->icon_size is in unscaled pixels.
        */
-      stage = clutter_actor_get_stage (CLUTTER_ACTOR (icon));
-      context = st_theme_context_get_for_stage (CLUTTER_STAGE (stage));
-      g_object_get (context, "scale-factor", &scale, NULL);
+      scale = st_theme_context_get_scale_for_stage ();
       new_size = (gint) (priv->theme_icon_size / scale);
     }
   else
