@@ -170,12 +170,11 @@ StThemeNode *         st_widget_get_theme_node            (StWidget        *widg
 StThemeNode *         st_widget_peek_theme_node           (StWidget        *widget);
 
 GList *               st_widget_get_focus_chain           (StWidget        *widget);
-void                  st_widget_paint_background          (StWidget        *widget);
+void                  st_widget_paint_background          (StWidget        *widget,
+                                                           ClutterPaintContext *paint_context);
 
 /* debug methods */
 char  *st_describe_actor       (ClutterActor *actor);
-void   st_set_slow_down_factor (gfloat factor);
-gfloat st_get_slow_down_factor (void);
 
 /* Compatibility methods */
 void  st_widget_destroy_children (StWidget     *widget);
